#
#	This file is part of the Tagore Desktop Wiki.
#
#	BrowserHistory
#
#	License: GPL
#
#	This class keeps track of the browser navigation history.
#

class BrowserHistory
	
	# Initializa the class.
	# Create a new array and set @history_index to 0.
	
	def initialize() 
		@history_index = 0 
		@browser_history = Array.new
	end 
	
	# Add new document to the history.
	# If the current document is not the last one in the history 
	# we cut the array and make the current document the last one.
	# Avoid to consecutive add the same document. 
	
	def add(document)
		if @browser_history.length != @history_index
        	@browser_history = @browser_history[0..@history_index - 1]
        end
        if document != @browser_history[@history_index - 1]
       		@history_index = @history_index + 1
	    	@browser_history.push(document)
	    end
	end
	
	# Delete a document from the history and rebuld the history
	# so we don't have the same document consecutively repeated
	
	def delete(document) 
		if @browser_history.length != @history_index
        	@browser_history = @browser_history[0..@history_index - 1]
        end
		@browser_history.delete(document)
		
		tmp_array = Array.new
		tmp_index = ''
		for index in @browser_history
			if index != tmp_index
				tmp_array.push(index)
				tmp_index = index
			end
		end
		@browser_history = tmp_array
		@history_index = @browser_history.length
	end

	# Go back in the history if possible and return the document name

	def back()
		if can_go_back?
			@history_index = @history_index - 1
			return @browser_history[@history_index - 1]
		end
	end 
	
	# Go forward in the history if possible and return the document name
	
	def forward()
		if can_go_forward?
			@history_index = @history_index + 1
			return @browser_history[@history_index - 1]		
		end
	end
	
	# Return true if we are not at the begining of the history
	
	def can_go_back? 
		(@history_index > 1)
	end
	
	# Return true if we are not at the end of the history
	
	def can_go_forward? 
		 (@history_index < @browser_history.length )
	end 
	
end