#
#	This file is part of the Tagore Desktop Wiki.
#
#	Storage
#
#	License: GPL
#

class Storage
    
    def initialize
    	@STORAGE_DIR  = ENV["HOME"] + "/.tagore/"
        @DATA_DIR      = @STORAGE_DIR + "data/"
        @TEMPLATES_DIR = @STORAGE_DIR + "templates/"
        
        @CURRENT_TEMPLATE = "default"
        

              
        if !FileTest.exists?(@STORAGE_DIR)
        	Dir.mkdir(@STORAGE_DIR)
            Dir.mkdir(@DATA_DIR)
            Dir.mkdir(@TEMPLATES_DIR)
            Dir.mkdir(@TEMPLATES_DIR + "default")
            copy_file(File.dirname($0) + "/templates/default.html", @TEMPLATES_DIR + "default/template.html")
            copy_file(File.dirname($0) + "/templates/style.css", @TEMPLATES_DIR + "default/style.css")
            copy_file(File.dirname($0) + "/templates/external.gif", @TEMPLATES_DIR + "external.gif")
            copy_file(File.dirname($0) + "/category.xml", @STORAGE_DIR + "category.xml")
            copy_file(File.dirname($0) + "/tagore.conf", @STORAGE_DIR + "tagore.conf")
        end
        
        @xmlfile = @STORAGE_DIR + "category.xml"
        @xmldocument = load_xml_document
        
    end
    
    # Load a xml document.
    # Used to load the tags and backlinks file.
    
    def load_xml_document()
    	file = File.new(@xmlfile)
        document = Document.new file
        file.close
        document
    end
    
    # Save a xml document.
    # Used to save the tags and backlinks file.
    
    def save_xml_document()
        file = File.new(@xmlfile, File::CREAT|File::TRUNC|File::WRONLY)
        @xmldocument.write(file,0)
        file.close  
    end
    
    # Returns the data dir path.
    
    def get_data_dir() 
    	@DATA_DIR 
    end 
    
    # Copy a file from _source_ to _target_
    
    def copy_file(source, target)
        source = File.open(source)
        target = File.open(target , "w")
        target.write( source.read(64) ) while not source.eof?
        source.close
        target.close    
    end
    
    # We are ready for document versioning.
    # Every time we edit a document a new file would be created
    # and its name saved in the _current_ file.
    
    def get_current_document(document)
    	document.downcase!    
        get_file_contents(@DATA_DIR + document + "/current").strip
    end
    
    # Returns a string with the file contents
    
    def get_file_contents(filename)
        File.open(filename){|f| ret = f.readlines.join }
    end
    
    # Returns a string with the source of _document_
    
    def get_document_source(document)
    	document.downcase!    
        get_file_contents(@DATA_DIR + document + "/revisions/" + get_current_document(document))
    end
    
    # Save the file _filename with the string _contents_
    
    def save_file_contents(filename, contents)
        File.open(filename, "w"){|f| f.write(contents)}
    end

	# Save the document _document_ with the contents of string _text_
	
    def save_document(document, text)
    	document.downcase!    
        delete_categories(document)
        a = text.scan(/^\[\[tags\:[\w+|\s|,]+\]\]$/)
        if a[0] !=nil
            save_categories(a, document)
        end
        a = text.scan(/\[\[[\w+|\s]+\]\]/)
        a.uniq!
        if a[0] !=nil
        	save_backlinks(a,document)
        end
                   
        save_file_contents(@DATA_DIR + document + "/revisions/"  + get_current_document(document) ,text)
    end
    
    # Save the tags on array _a_ for the named document
    
    def save_categories(a, document)
        a = a[0][7..-3].strip
        a = a.split(",")
        for x in a
            x.split
            add_category(document, x)
        end
        save_xml_document
    end
    

    # check if a document exists.
    # When revisions feature is implemented we should look if
    # the directory exists, instead to look to the file.
    
    def file_exists(filename)
    	filename.downcase!
        FileTest.exists?(@DATA_DIR + filename + "/revisions/00000001") 
    end
    
    # delete a document and its dirs
    
    def delete_document(document)
    	document.downcase!    
        File.delete(@DATA_DIR + document + "/revisions/"  + get_current_document(document))
        File.delete(@DATA_DIR + document + "/current")
        Dir.delete(@DATA_DIR + document + "/revisions/")
        Dir.delete(@DATA_DIR + document)
        delete_categories(document)
    end
    
    # returns the time with format: day_name day_number, month year. (hour:minutes)
    
    def get_mtime(document)
    	document.downcase!    
        if FileTest.exists?(@DATA_DIR + document + "/revisions/"  + get_current_document(document))
            mtime = File.mtime(@DATA_DIR + document + "/revisions/" + get_current_document(document)).strftime("%A %d, %B %Y. (%H:%M)")
        else
            mtime = ""
        end
        mtime
    end
    
    # create a new document.
    # Each document is saved in a directory with the document name,
    # In this directory a new directory _revisions_ where all document
    # revisions are going to be stored.
    # The file _current_ has a reference to the current revision of
    # the document, which is 00000001 at first.
    
    def new_document(document)
    	document.downcase!
        Dir.mkdir(@DATA_DIR + document)
        Dir.mkdir(@DATA_DIR + document + "/revisions")
        f = File.new(@DATA_DIR + document + "/current", File::CREAT|File::EXCL|File::WRONLY)
        f.write("00000001")
        f.close
        f = File.new(@DATA_DIR + document + "/revisions/00000001", File::CREAT|File::EXCL)
        f.close
    end
    
    # Return a sorted list with the contents of the array list_array
    
    def sorted_list(list_array)
        list_array.sort!
        s = ""
        first_letter  = ""
        for a in list_array 
	        if a[/^./].upcase != first_letter
	        	first_letter = a[/^./].upcase
	        	s = s + "\nh1. " + first_letter + "\n"
	        end
	        s = s + "* [["+a+"]]\n"	        	
        end
        s  	    	 
    end
    
    # Return a list with links to all documents
    
    def get_all_documents
        d = Dir.entries(@DATA_DIR)
        d.delete(".")
        d.delete("..")
		s = sorted_list(d)
        generate_document(s, "Documents", Time.now.strftime("%A %d. %B %Y. (%H:%M)"))
    end
  
  	# get the page's tags
  	
    def get_document_tags(a, r)
    	b = a[7..-3].strip
        b = b.split(",")
        e = ""
        for x in b
	        x = x.strip.downcase
	        e = e + create_wikilink("[[category:"+x+"]]")
        end
        r.gsub(a.to_s,"<div class=\"tags\">" + e + "</div>")
      end          
              
    # Generate the html document with contents in _string_
    
      def generate_document(string, title = "", mtime = "")    
          r = RedCloth.new string
            
            # tags
            a = r.scan(/^\[\[tags\:[A-Za-zá-ú\s|,]+\]\]$/)
            if a[0] != nil
                r = get_document_tags(a[0],r)
            end
            
          # category links  
          a = r.scan(/\[\[category\:[A-Za-zá-ú\s]+\]\]/)
          a.uniq!
          for z in a 
              r = r.gsub(z, create_wikilink(z))
          end
          
           # wikilinks
           a = r.scan(/\[\[[A-Za-zá-ú\s]+\|?[A-Za-zá-ú\s]+\]\]/)
           a.uniq!
           for x in a
               r = r.gsub(x,create_wikilink(x))
           end
           
           template = get_file_contents(@TEMPLATES_DIR + @CURRENT_TEMPLATE + "/template.html")

           template = template.gsub("@PAGENAME@",title)
           template = template.gsub("@STYLESHEET@",@TEMPLATES_DIR + @CURRENT_TEMPLATE + "/style.css")
           template = template.gsub("@MODDATE@",mtime)
           template = template.gsub("@CONTENT@",r.to_html)        
           template
      end

      # creates internal wiki links
      
      def create_wikilink(link)
          linktext = link = link[2..-3]
          if link.index("|") != nil
              lsplit = link.split("|",2)
              linktext = lsplit[1]
              link = lsplit[0]
          end
          if file_exists(link)
              htmlclass = "wikilink"
          else
              htmlclass = "nodocument"
          end
          if link.index("category:") == 0
              htmlclass = "tag"
              linktext = link.split(":")[1].capitalize
          end   
          "<a href=\"" + link + "\" class=\"" + htmlclass + "\">"+ linktext.capitalize + "</a>"
      end
  
      # return the document parsed to html.
      
      def get_document(document)
          if document == "@@ALL_DOCUMENTS@@"
              get_all_documents
          elsif document == "@@ALL_CATEGORIES@@"
              get_all_categories
          elsif document.index("category:") == 0
              lsplit = document.split(":",2)
              get_category(lsplit[1].strip)
           else
               mtime = get_mtime(document)
               s = get_document_source(document)
               s = s + get_backlinks(document)
               generate_document(s, document, mtime)
           end
       end
       
       # export document to html file.
       # In the future we should be able to export multiple files and 
       # full tags, so this method is going to be reworked.
       
    def export_document(document, filename)
             
       	mtime = get_mtime(document)
       	string = get_document_source(document)
       	r = RedCloth.new string
       	#string = generate_document(s, document, mtime)
       	template = get_file_contents(@TEMPLATES_DIR + @CURRENT_TEMPLATE + "/template.html")

        template = template.gsub("@PAGENAME@",document)
        template = template.gsub("@STYLESHEET@","css/style.css")
        template = template.gsub("@MODDATE@",mtime)
        template = template.gsub("@CONTENT@",r.to_html)        
           
		save_file_contents(filename, template)       	
		
		Dir.mkdir(File.dirname(filename) + "/css")
		d = Dir.entries(@TEMPLATES_DIR + @CURRENT_TEMPLATE)
        d.delete(".")
        d.delete("..")
        d.delete("template.html")
		for a in d
			copy_file(@TEMPLATES_DIR + @CURRENT_TEMPLATE + "/" + a, File.dirname(filename) + "/css/" + a)
		end
		
    end
       
    # Save the backlinks
       
    def save_backlinks(array, document) 
		for index in array
			index.downcase.split
	        document.downcase!
          
          	index = index[2..-3]
          	if index.index("|") != nil
              lsplit = index.split("|",2)
              index = lsplit[0]
          	end
          	        
	        element = @xmldocument.root.elements["document[@name='" + document +"']"]
	        if element == nil
	            element = @xmldocument.root.add_element "document", {"name"=>document}
	        end
	        element.add_element "link", {"name"=>index}
            save_xml_document	        			
		end  	 
    end        
       
     # Get the backlinks for a document.
     
     def get_backlinks(document) 
        e = Array.new
        @xmldocument.elements.each("tagore/document/link[@name='"+document+"']") { |element| e.push "[[" + element.parent.attributes["name"] +"]]"}
        e.uniq!
        e.sort!
        string = ""
        for x in e
            string = string + x
        end
        if string != ""
        	string = "\n<div class=\"backlinks\">" + string + "</div>"
        else
        	string = ""
        end
        string    	 
     end   
     
     # Return a page listing all avaliable categories / tags.
     
     def get_all_categories
        e = Array.new
        @xmldocument.elements.each("tagore/document/category") { |element| e.push "[[category:" + element.attributes["name"] +"]]"}
        e.uniq!
        e.sort!
        string = ""
        for x in e
            string = string + x
        end
        string = "<div class=\"cloud\">" + string + "</div>"
        generate_document(string, "Tags", Time.now.strftime("%A %d. %B %Y. (%H:%M)"))
     end
     
     # return the documents in a tag
     
     def get_category(category)
     	category.downcase!
        e = Array.new
        @xmldocument.elements.each("tagore/document/category[@name='"+category+"']") { |element| e.push(element.parent.attributes["name"])}
        e = sorted_list(e)
        generate_document(e, "Tag:"+category, Time.now.strftime("%A %d. %B %Y. (%H:%M)"))         
     end
     

	# Add a tag to a document.
     
     def add_category(document, category)
        category=category.downcase.strip
        document.downcase!
     
        element = @xmldocument.root.elements["document[@name='" + document +"']"]
        if element == nil
            element = @xmldocument.root.add_element "document", {"name"=>document}
        end
            
        element.add_element "category", {"name"=>category}
     end
     
     # delete tags and backlinks for a document.
     # This method should be renamed.
     
     def delete_categories(document)
     	document.downcase!
        @xmldocument.delete_element "tagore/document[@name='" + document +"']"
        save_xml_document
     end
        
     # Save the window postition and size so we can restore it.   
        
     def save_window_position(window_settings)
     	string = window_settings[0].to_s + "\n"
     	string = string + window_settings[1].to_s + "\n"
     	string = string + window_settings[2].to_s + "\n"
     	string = string + window_settings[3].to_s + "\n"
     	string = string + window_settings[4].to_s + "\n"
     	save_file_contents(@STORAGE_DIR + "tagore.conf", string)     	 
     end 
     
     # Read the window position and size and return it all in an array
     
     def get_window_position
     	c = Array.new
     	conf = get_file_contents(@STORAGE_DIR + "tagore.conf")
     	conf.each {|line| c.push(line)}
     	c
     end
         
end
