#!/usr/bin/env ruby
#
#	Tagore Desktop Wiki
#
#	License: GPL	
#

require 'libglade2'
require 'gtkmozembed'
require 'lib/redcloth'
require 'lib/storage'
require 'lib/browserhistory'
require "rexml/document"
require 'gtktrayicon'
include REXML
         
class WikirubyGlade
  include GetText

  attr :glade
  
  def initialize(path_or_data, root = nil, domain = nil, localedir = nil, flag = GladeXML::FILE)
    bindtextdomain(domain, localedir, nil, "UTF-8")
    @glade = GladeXML.new(path_or_data, root, domain, localedir, flag) {|handler| method(handler)}
    @gecko = @glade["gecko"]
    @notebook = @glade["notebook"]
    @sourceview = @glade["sourceview"]
    @toolbar1 = @glade["toolbar1"]
    @editbutton = @glade["editbutton"]
    @enganxa = @glade["enganxa1"]
    @copia = @glade["copia1"]
    @retalla = @glade["retalla1"]
    @suprimeix = @glade["suprimeix1"]
    @dialog = @glade["dialog"]
    @dialogtext = @glade["dialogtext"]
    @about = @glade["about"]
    @backbutton = @glade["backbutton"]
    @forwardbutton = @glade["forwardbutton"]
    @categorydialog = @glade["categorydialog"]
    @categorycombo = @glade["categorycombo"]
    @window = @glade["window"]
    @filedlg = @glade["filedialog"]
    @export_menu = @glade["export_document"]
    @bookmarks = @glade["bookmarks"]
    
    @LINK_MSG = ""
    @filename = ""
    @storage  = Storage.new
	@history = BrowserHistory.new

	# Setup the TrayIcon
	
 	tray = Gtk::TrayIcon.new('Tagore')
    event_box = Gtk::EventBox.new
    image = Gtk::Image.new
    tray.add event_box
    event_box.add image
	size = 0
	
	event_box.signal_connect("button_press_event") do
  		@window.visible = !@window.visible?
	end
	
 	tray.signal_connect("configure_event") do |widget, event|
 		size = event.width > event.height ? event.width : event.height
		nil		# it doesn't work without this! (?)
 	end	

 	tray.signal_connect("map_event") do
	  		pixbuf = Gdk::Pixbuf.new("tagore.svg",21,21)
			image.pixbuf = pixbuf
	end
 	tray.show_all

	# Restore the last window size and position
	
 	conf = @storage.get_window_position
 	@window.resize(conf[0].strip.to_i,conf[1].strip.to_i)
 	@window.move(conf[2].strip.to_i,conf[3].strip.to_i)
 	if conf[4].strip.to_i == 1
 		@window.maximize 
 	end   	 
 	
 	# First page is @@ALL_CATEGORIES@@
 	
    get_all_tags
    check_browser_buttons 	
 	on_edit_document
 	editable_document

  end
  
  # Display a list with all avaliable tags in the browser
  
  def get_all_tags
       open_page(@storage.get_all_categories)
       editable_document
       @filename = "@@ALL_CATEGORIES@@"
       @history.add(@filename)
       check_browser_buttons
  end

  # Display a list with all avaliable documents in the browser
  
  def get_all_documents
      open_page(@storage.get_all_documents)
      @filename = "@@ALL_DOCUMENTS@@"
	  @history.add(@filename)
      editable_document
      check_browser_buttons
  end
    
  # On browser's back button clicked we go back in history,
  # display the new page, refresh the browser back and forward buttons
  # and enable or disable the edit buttons
  
  def on_backbutton_clicked(widget)
 		@filename = @history.back
        refresh_browser
        check_browser_buttons
        editable_document
  end

  # On browser's forward button clicked we go back in history,
  # display the new page, refresh the browser back and forward buttons
  # and enable or disable the edit buttons
      
  def on_forwardbutton_clicked(widget)
		@filename = @history.forward
        refresh_browser
        check_browser_buttons
        editable_document
  end
  
  # Check if the current page is editable and enable or disable
  # the edit button and delete option in edit menu.
  
  def editable_document
  	editable = !(@filename.index("@@") || @filename.index("category"))
    @editbutton.sensitive = editable
    @suprimeix.sensitive  = editable
    @export_menu.sensitive = editable    
  end
  
  # On browser view we disable the copy/cut/paste options and enable delete
  # This options are only enabled in edit mode.
  
  def on_edit_document
      bool = !(@notebook.page == 0)
      @enganxa.sensitive   = bool
      @copia.sensitive     = bool
      @retalla.sensitive   = bool
      @suprimeix.sensitive = !bool
      @bookmarks.sensitive = !bool
  end
  
  # Export the current document to html file
  
  def export_document()
	ret = @filedlg.run
	if ret == Gtk::Dialog::RESPONSE_OK  	
  		@storage.export_document(@filename, @filedlg.filename)
  	end
  	@filedlg.hide
  end
  
  # Enable or disable browser back and forward buttons based on history
    
  def check_browser_buttons
      @forwardbutton.sensitive 	= @history.can_go_forward?
      @backbutton.sensitive 	= @history.can_go_back? 
  end
  
  # Delete current document and display a list with all documents available
  
  def on_suprimeix1_activate(widget)
    @storage.delete_document(@filename)
    @history.delete(@filename)
    get_all_documents
  end

  # Create new document, if document exists just edit it.
  
  def new_page(widget)
	ret = @dialog.run
	if ret == Gtk::Dialog::RESPONSE_OK
    	if(@storage.file_exists(@dialogtext.text))
        	@filename = @dialogtext.text
            on_editbutton_clicked(widget)
        else
        	@storage.new_document(@dialogtext.text)
            @filename = @dialogtext.text
            @sourceview.buffer.text = ""
            @toolbar1.sensitive = false
            @notebook.page = 1
        end
		@history.add(@filename)
	    check_browser_buttons                
	end
    @dialog.hide
    @dialogtext.text = ""
  end

  # Cut/Copy/Paste options only in edit mode
  
  def on_enganxa1_activate(widget)
      if @notebook.page == 1 
          @sourceview.signal_emit("paste_clipboard")
      end
  end  
  def on_retalla1_activate(widget)
      if @notebook.page == 1 
          @sourceview.signal_emit("cut_clipboard")
      end
  end
  def on_copia1_activate(widget)
      if @notebook.page == 1 
          @sourceview.signal_emit("copy_clipboard")
      end
  end
  
  # When home button is clicked we display a list with all categories.
  
  def on_homebutton_clicked(widget)
    get_all_tags
  end
  
  # Get the document's source, go to edit mode and enable menu options.
  
  def on_editbutton_clicked(widget)
        @sourceview.buffer.text = @storage.get_document_source(@filename)
        @toolbar1.sensitive = false
        @notebook.page = 1
        on_edit_document
  end
  
  # Cancel button on the edit view is clicked
  
  def on_cancelbutton_clicked(widget)
    @notebook.page = 0
    refresh_browser
    @sourceview.buffer.text = ""
    @toolbar1.sensitive = true
    on_edit_document
  end
  
  # Save document and refresh the browser so we can see the changes
  
  def on_savebutton_clicked(widget)
      @storage.save_document(@filename,@sourceview.buffer.text)
      refresh_browser      
      @notebook.page = 0
      @sourceview.buffer.text = ""
      @toolbar1.sensitive = true
      on_edit_document
  end
  
  # Show the 'about...' window
  
  def on_quant_a1_activate(widget)
    @about.run
    @about.hide
  end
  
  # TODO: display link target in the statusbar
  
  def on_link_message(widget)
      @LINK_MSG = @gecko.link_message
  end
  
  # When user clicks on a link we look if it is an internal or external link
  
  def on_open_uri(widget, uri)
      if @LINK_MSG.index("category:") == 0
          @filename = @LINK_MSG
          refresh_browser
		  @history.add(@filename)
          check_browser_buttons
          editable_document
      elsif @LINK_MSG.index("file://") == 0
          if @storage.file_exists(File.basename(@LINK_MSG))
              @filename = File.basename(@LINK_MSG)
              refresh_browser
          else
              @storage.new_document(File.basename(@LINK_MSG))
              @filename = File.basename(@LINK_MSG)
              @sourceview.buffer.text = ""
              @toolbar1.sensitive = false
              @notebook.page = 1
          end
		  @history.add(@filename)
          check_browser_buttons
      else
          system("gnome-open " + @LINK_MSG)
      end
      true
  end
   
  # Refresh the browser so we can update the document
  
  def refresh_browser 
    text = @storage.get_document(@filename)
    open_page(text)
  end
  
  # Open a document in the browser view
  
  def open_page(string)
    @gecko.open_stream("file:/"+@storage.get_data_dir,"text/html")
    @gecko.append_data(string)
    @gecko.close_stream
    editable_document
  end
  
  # Exit the program 
  
  def on_quit(widget, arg0=nil)
  	window_settings = Array.new
  	window_settings[0] = @window.size[0]
  	window_settings[1] = @window.size[1]
  	window_settings[2] = @window.position[0]
	window_settings[3] = @window.position[1]
	if @window.window.state & Gdk::EventWindowState::MAXIMIZED == Gdk::EventWindowState::MAXIMIZED then
	window_settings[4] = 1
	else
	window_settings[4] = 0
	end
  	@storage.save_window_position(window_settings)  	
    Gtk.main_quit
  end
    
end

# Main program

if __FILE__ == $0
  # Set values as your own application. 
  PROG_PATH = "wikiruby.glade"
  PROG_NAME = "Tagore"
  Gtk.init
  WikirubyGlade.new(PROG_PATH, nil, PROG_NAME)
  Gtk.main
end
